﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TSLab.Script.Handlers;

namespace SomeIndicators
{

    public class AMA : IDouble2DoubleHandler
    {
        [HandlerParameter(true, "20", Min = "10", Max = "100", Step = "5")]
        public int Period { get; set; }

        public IList<double> Execute(IList<double> source)
        {
            var count = source.Count;

            var res = new double[count];
            if (Period > 0 && count > 0)
            {
                for (int k = 0; k <= Period; k++)
                {
                    res[k] = source[k];
                }
                var ama0 = count < Period ? 0 : source[Period + 1];
                for (int k = Period + 2; k < count; k++)
                {
                    var signal = Math.Abs(source[k] - source[k - Period]);
                    var noise = 0.000000001;
                    for (int i = 0; i < Period; i++)
                    {
                        noise = noise + Math.Abs(source[k - i] - source[k - i - 1]);
                    }
                    var er = signal / noise;
                    var ssc = (er * 0.60215) + 0.06452;
                    var cst = Math.Pow(ssc, 2);
                    var ama = ama0 + cst * (source[k] - ama0);
                    res[k] = ama;
                    ama0 = ama;
                }
            }
            return res;
        }
    }


    public class AMkAbase
    {
        [HandlerParameter(true, "9", Min = "2", Max = "20", Step = "1")]
        public int PeriodAMA { get; set; }

        [HandlerParameter(true, "2", Min = "2", Max = "20", Step = "1")]
        public double NFast { get; set; }

        [HandlerParameter(true, "30", Min = "5", Max = "50", Step = "5")]
        public double NSlow { get; set; }

        [HandlerParameter(true, "2", Min = "1", Max = "5", Step = "0.5")]
        public double Pow { get; set; }

        [HandlerParameter(true, "1", Min = "1", Max = "5", Step = "0.5")]
        public double dK { get; set; }

        void InsertDif(IList<double> ddAMA, double a)
        {
            //---- проверка, заполнен ли массив
            for (int i = 0; i < PeriodAMA; i++) //для всех элементов массива
            {
                if (ddAMA[i] == 0) //если элемент равен нулю
                {
                    ddAMA[i] = a; //сохраняем значение в этот элемент
                    return; //и удачно завершаемся
                }
            }
            //---- массив уже заполнен, нужно вставлять элемент с конца
            for (int i = 0; i < PeriodAMA - 1; i++) //все элементы массива, кроме последнего
            {
                ddAMA[i] = ddAMA[i + 1]; //сдвигаем влево на одну позицию
            }
            ddAMA[PeriodAMA - 1] = a; //и записываем значение в самую правую позицию
            return; //после чего удачно завершаемся
        }

        public IList<double> Execute(IList<double> source, int resType)
        {
            var count = source.Count;

            var slowSC = (2.0 / (NSlow + 1)); //медленный к-т сглаживания
            var fastSC = (2.0 / (NFast + 1)); //быстрый к-т сглаживания

            var kAMAbuffer = new double[count];
            var kAMAupsig = new double[count];
            var kAMAdownsig = new double[count];
            
            var ddAMA = new double[PeriodAMA];

            if (PeriodAMA > 0 && count > 0)
            {
                for (int k = 0; k < PeriodAMA + 2; k++)
                {
                    kAMAbuffer[k] = source[k];
                }
                var ama0 = count < PeriodAMA + 2 ? 0 : source[PeriodAMA + 1];
                for (int k = PeriodAMA + 2; k < count; k++)
                {
                    //---- расчёт сигнала
                    double signal = Math.Abs(source[k] - source[k - PeriodAMA]);
                    //---- расчёт шума
                    double noise = 0.000000001;
                    for (int i = 0; i < PeriodAMA; i++)
                    {
                        noise = noise + Math.Abs(source[k - i] - source[k - i - 1]);
                    }
                    //---- расчёт коэффициента сглаживания
                    double ER = signal / noise; //коэффициент эффективности
                    double SSC = ER * (fastSC - slowSC) + slowSC; //коэффициент сглаживания
                    //---- расчёт главной линии
                    double AMA = ama0 + Math.Pow(SSC, Pow) * (source[k] - ama0); //расчёт
                    kAMAbuffer[k] = AMA; //вывод
                    //---- расчёт фильтрации тренда
                    double ddK = (AMA - ama0)/1; // Point; //разность

                    InsertDif(ddAMA, ddK); //накапливаем приращение

                    //---- расчёт среднего арифметического
                    double SMAdif = 0; //вначале равно нулю
                    for (int i = 0; i < PeriodAMA; i++)
                    {
                        SMAdif += ddAMA[i]; //последовательно суммируем
                    }
                    SMAdif /= PeriodAMA; //и делим на количество
                    //---- расчёт стандартного отклонения
                    double StDev = 0; //вначале равно нулю
                    for (int i = 0; i < PeriodAMA; i++)
                    {
                        StDev += Math.Pow(ddAMA[i] - SMAdif, 2); //суммируем квадраты отклонений
                    }
                    StDev = Math.Sqrt(StDev) / PeriodAMA; //извлекаем корень и делим на количество
                    //---- расчёт фильтра
                    double Filter = dK * StDev;


                    //---- обработка значений
                    double var1 = 0, var2 = 0;
                    if (ddK > Filter)
                        var1 = AMA; //есть восходящий тренд
                    if (ddK < -Filter)
                        var2 = AMA; //есть нисходящий тренд
                    kAMAupsig[k] = var1; //нет восходящего тренда
                    kAMAdownsig[k] = var2; // нет нисходящего тренда

                    ama0 = AMA;
                }
            }
            if(resType == 3)
            {
                for(int i = 0; i < count; i++)
                {
                    if(kAMAdownsig[i] > 0)
                    {
                        kAMAupsig[i] = -kAMAdownsig[i];
                    }
                }
                return kAMAupsig;
            }
            return resType == 0 ? kAMAbuffer : resType == 1 ? kAMAupsig : kAMAdownsig;
        }
    }

    public class AMkA : AMkAbase, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Execute(source, 0);
        }
    }

    public class AMkAUp : AMkAbase, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Execute(source, 1);
        }
    }

    public class AMkADown : AMkAbase, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Execute(source, 2);
        }
    }

    public class AMkASignal : AMkAbase, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Execute(source, 3);
        }
    }
}
