﻿using System;
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;

namespace SomeIndicators
{
    [HandlerName("Last Closed Position Time")]
    public class LastClosedPositionTime : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.LastPositionClosed;
            if(pos != null && pos.ExitBarNum > barNum)
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.EntryBar.Date;
            return dt.Hour*10000.0 + dt.Minute*100.0 + dt.Second;
        }
    }

    [HandlerName("Last Closed Position Date")]
    public class LastClosedPositionDate : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.LastPositionClosed;
            if (pos != null && pos.ExitBarNum > barNum)
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.EntryBar.Date;
            return (dt.Year % 100) * 10000.0 + dt.Month * 100.0 + dt.Day;
        }
    }

    [HandlerName("Last Closed Position Exit Time")]
    public class LastClosedPositionExitTime : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.LastPositionClosed;
            if (pos != null && pos.ExitBarNum > barNum)
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.ExitBar.Date;
            return dt.Hour * 10000.0 + dt.Minute * 100.0 + dt.Second;
        }
    }

    [HandlerName("Last Closed Position Exit Date")]
    public class LastClosedPositionExitDate : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.LastPositionClosed;
            if (pos != null && pos.ExitBarNum > barNum)
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.ExitBar.Date;
            return (dt.Year % 100) * 10000.0 + dt.Month * 100.0 + dt.Day;
        }
    }

    [HandlerName("Last Closed Named Position Exit Time")]
    public class LastClosedNamePositionExitTime : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        [HandlerParameter(true, "", NotOptimized = true)]
        public string Name { get; set; }

        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.GetLastForSignal(Name);
            if (pos != null && (pos.IsActive || pos.ExitBarNum > barNum))
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.ExitBar.Date;
            return dt.Hour * 10000.0 + dt.Minute * 100.0 + dt.Second;
        }
    }

    [HandlerName("Last Closed Named Position Exit Date")]
    public class LastClosedNamePositionExitDate : IOneSourceHandler, IDoubleReturns, IValuesHandler, ISecurityInputs
    {
        [HandlerParameter(true, "", NotOptimized = true)]
        public string Name { get; set; }

        public double Execute(ISecurity source, int barNum)
        {
            var pos = source.Positions.GetLastForSignal(Name);
            if (pos != null && (pos.IsActive || pos.ExitBarNum > barNum))
            {
                pos = null;
            }
            var dt = pos == null ? DateTime.MinValue : pos.ExitBar.Date;
            return (dt.Year % 100) * 10000.0 + dt.Month * 100.0 + dt.Day;
        }
    }

    [HandlerName("Days In Position")]
    public class DaysInPosition : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            var dt = pos.EntryBar.Date;
            var curDt = pos.Security.Bars[barNum].Date;
            return (curDt-dt).TotalDays;
        }
    }

    [HandlerName("Minutes In Position")]
    public class MinutesInPosition : IPosition2Double
    {
        public double Execute(IPosition pos, int barNum)
        {
            if (pos == null)
            {
                return 0;
            }
            var dt = pos.EntryBar.Date;
            var curDt = pos.Security.Bars[barNum].Date;
            return (curDt - dt).TotalMinutes;
        }
    }
}
